﻿/*

File:        lr.d

Purpose:     lr(1)/lalr(1)/glr parser

Copyright (c) 2004, Ivan Senji
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

	*	Redistributions of source code must retain the above copyright notice, this
		list of conditions and the following disclaimer. 
	*	Redistributions in binary form must reproduce the above copyright notice, 
		this list of conditions and the following disclaimer in the documentation 
		and/or other materials provided with the distribution. 
	*	Neither the name of the <ORGANIZATION> nor the names of its contributors 
		may be used to endorse or promote products derived from this software 
		without specific prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
	POSSIBILITY OF SUCH DAMAGE.

*/

//probam napraviti promjenu da napravim još jednu pa vidim šta će bit

// još par promjena

module lr;

private
{
	import std.c.stdio, std.stdio;
	import std.c.time;
	import std.conv;
	import std.stream;
	import std.string;
	import ivan_set;
	import ivan_nodetree;
	import std.thread;
	import std.c.windows.windows;
	import std.zlib;
	import std.stdarg;
	import std.file;
  import std.math;
}

//version = GrammarTest;
//version = noviParse;
version = zippedTableFile;
//version = enableHTMLoutput;

/***********************************************************************

		Some aliases for set's used and the ILENode where the resulting 
		parse tree is used

***********************************************************************/

alias set!(int) IntegerSet;
alias set!(GrammarRule) GrammarRuleSet;
alias set!(State) StateSet;
alias set!(Stavka) StavkaSet;
alias set!(char[]) CharArraySet;
alias set!(RowElement) RowElementSet;
alias Node!(ILexElement) ILENode;

private alias char[] string;

typedef RowElementSet[][] Table;

template minT(A)
{
  A min(A a, A b)
  {
    return a <= b ? a : b;
  }
}
alias minT!(int).min min;
/***********************************************************************

		template for casting array of objects of type A to interface 
		type B. Used when calling Parser.parse(ILexElement[])

***********************************************************************/

template castA2I(A,B)
{
	B[] castA2I(A[] input)
	{
		B[] b = new B[input.length];
		foreach(int index,A a;input)
		{
			b[index] = cast(B)a;
		}
		return b;
	}
}

char[] IntToString(int i)
{
	return std.string.toString(cast(uint)i);
}

private enum Special{EOF = int.min, TOCKA = int.min+1, REDUCTION = -1, ACCEPT = -2, ERROR = int.min+1 };

/***********************************************************************

		class containing information parser needs to work, some things
		need to be removed

***********************************************************************/

class Setup
{
	public:
	char[] inputfile;
	char[] testfile;	//remove
	int testFileLoadMethod = 0;	//remove
	bool parserOutput = false;
	bool printProductions = false;

	bool printNez = false;
	bool printZav = false;

	bool printParseTree2XML = false;
	bool forseRecalc = false;
	bool lalr = false;
	int lalrExtraPasses = 0;
}

/**********************************************************************

		interface wich a class (containing lexical elements (or tokens)) 
		must implement so it can be used as parser input. The 
		implementations of the 2 methods must look something like this:

		int getIndex()
		{
			return G.getIndex(this.zn);
		}
		ILexElement create(char[] name)
		{
			return new MyLex(name);
		}
		
		here this.zn is of type char[] and matches the terminal in a 
		grammar

***********************************************************************/

interface ILexElement
{
	int getIndex();
	ILexElement create(char[] name);
}

/***********************************************************************

		A single rule, used by Grammar(set of rules) and by Parser to 
		create the parsing table

***********************************************************************/

private class GrammarRule
{	
  private this(){dotPosition = -1; }
	
	private this(int l, int[] r)
	{
		left = l;
		right = r;
		dotPosition = -1;
	}

	private static GrammarRule newGrammarRuleWithDot(GrammarRule p, int posTocka)
		in{assert(posTocka >= 0 && posTocka <= p.right.length);}
		out(nova){assert(nova.right.length == p.right.length || nova is null);}
	body
	{
		if(posTocka <= p.right.length)
		{
			GrammarRule newRule = new GrammarRule();
			{
				newRule.left = p.left;
				newRule.right = p.right.dup;
				newRule.dotPosition = posTocka;
			}
			return newRule;
		}
		else return null;
	}
	char[] toStringNumbers(Grammar G)
	{
		char[] rez;
		rez ~= std.string.toString(left);
		rez ~= " -> ";
		foreach(int index,int znak; right)
		{
			rez ~= std.string.toString(znak);
			rez ~= " ";
		}
		return rez;
	}
	char[] toString(Grammar G)
	{
		char[] rez;
		rez ~= G.nonTerminalToString[left];
		rez ~= " -> ";
		foreach(int index,int zn;right)
		{
			if(index == dotPosition)
				rez ~= G.terminalToString[Special.TOCKA] ~ " ";

			if(zn<0)
			{
				rez ~= G.terminalToString[zn];
			}

			else if(zn >= 0)
			{
				rez ~= G.nonTerminalToString[zn];
			}
			rez ~= " ";
		}
		if(right.length == dotPosition)
			rez ~= G.terminalToString[Special.TOCKA] ~ " ";
		return rez;
	}

	version(enableHTMLoutput)
	{
		void toHTML(Stream file,Grammar G)
		{
			file.printf("&lt%.*s&gt -> ",G.nonTerminalToString[left][1..G.nonTerminalToString[left].length-1]);
			foreach(int index,int zn;right)
			{
				if(index == dotPosition)
					file.printf(" @ ");
	
				if(zn<0)
				{
					file.printf("%.*s ",G.terminalToString[zn]);
				}
	
				else if(zn >= 0)
				{
					file.printf("&lt%.*s&gt ",G.nonTerminalToString[zn][1..G.nonTerminalToString[zn].length-1]);
				}
			}
			if(right.length == dotPosition)
				file.printf(" @ ");
		}
	}
	
	bool isEpsilon()
	{
		return ( right.length == 0 /*&& (dotPosition == -1 || dotPosition == 0 ) */);
	}

	bool hasDot()
	{
		return dotPosition != -1;
	}

	bool nonTerminalAfterDot()
	in{
		assert(this.hasDot());
		assert(this.PolozajTocke <= this.right.length);
	}
	body
	{
		if(right[dotPosition] >= 0)return true;
		else return false;
	}

	int ZnakIzaTocke()
	{
		return this.right[this.PolozajTocke];
	}

	int PolozajTocke()
	in{
		assert(this.hasDot());
	}
	out(result){assert(result>=0);}
	body
	{
		return dotPosition;
	}

	int opEquals (Object o)
	{
		GrammarRule	p2 = cast(GrammarRule)o;
		if(this.left != p2.left) return false;
		if(this.right.length != p2.right.length) return false;
		for(int i=0; i<this.right.length; i++)
		{
			if(this.right[i]!=p2.right[i])return false;
		}
		if(this.dotPosition != p2.dotPosition && (this.dotPosition!=-1 && p2.dotPosition!=-1))return false;
		return true;
	}

	int opCmp ( Object o)
	{
		GrammarRule druga =cast(GrammarRule)o;
		return this.toHash < druga.toHash;
	}
  
	//maybe don't need this
	uint toHash()
	{
		int br = this.left;
		int indx=10;
		foreach(int rbr,int x; this.right)
		{
			br += ((x<0?(-x*2):(2*x+1))+1)*((x<0?(-x*2):(2*x+1))+1)*indx;
			indx*=1009;
		}
		return br;
	}
  
	bool SviNezavrsni()
	{
		if(right.length == 0)return false;
		foreach(int elem; right)
		{
			if(elem<0)return false;
		}
		return true;
	}

	static bool Cmp(GrammarRule p1, GrammarRule p2)
	{
		if(p1.left != p2.left) return false;
		if(p1.right.length != p2.right.length) return false;
		if(p1.right[]!=p2.right[])return false;
		return true;
	}

	int length(){return right.length;}

	int at(int pos) in {pos<=length;}
	body
	{
		return right[pos];
	}

	private int left;
	private int[] right;
	private short dotPosition;

	unittest
	{
		int[] x = new int[10];
		int[] y = new int[10];

		for(int i=0; i<x.length; i++)
			x[i]=y[i]=i;

		GrammarRule p1 = new GrammarRule(0,x);
		GrammarRule p2 = new GrammarRule(0,y);
		assert(p1 == p2);
		assert(!(p1 is p2));
		GrammarRule p3 = newGrammarRuleWithDot(p1,3);
		assert(p1 != p3);
		assert(Cmp(p1,p3)==true);
		printf("GrammarRule.Cmp unittest passed!\n");
	}
}


/***********************************************************************

		A class describing code segments attached to a grammar rule

***********************************************************************/

private class GrammarRuleCode
{
	string arguments;
	string[][] parts;
	string[] calls;
}

/*
	terminals <0
	nonterminal >=0
*/
class Grammar
{
	class GrammarException:Exception
	{
		this(char[]msg)
		{
			super(msg);
		}
	}

	public this(Setup s)
	{
		debug writefln("loading grammar file!");
	
		GlobalSetup = s;
		clock_t pocetak = clock();
		File file = new File(GlobalSetup.inputfile,FileMode.In);
		grammarRules = new GrammarRuleSet();

		State.G = this;
		Transititon.G = this;

		char[] word;
    char[] arguments;

		int productionNO = 0;
		
		nonTerminalToInt["<S_>"] = nonTerminalToInt.length;
		parsersFirstNonTerminal = nonTerminalToInt["<S_>"];
	
		GrammarRule previousRule = null;
    GrammarRuleCode grc;

    int left;
    int[] right;
    string[] dijeloviProdukcije;
    grc = new GrammarRuleCode;
		do
		{
		  file.readf("%s",&word);
		  debug writef("%s ",word);
      if(word.length == 0)break;
        
      right.length = 0;
      
      //ako je sam početak, a nalazi se {{ znači da je kod main dijela
      if(previousRule is null && word == "{{")
      {
        debug writefln("Nasao kod main dijela");
        while(word = file.readLine(), word != "}}")
        {
          debug writefln("%s",word);
          mainCode ~= word ~ "\n";
        }
        debug writefln("Gotov kod main dijela");

        //pročitaj slijedeću riječ
        file.readf("%s",&word);
        debug writef("%s ",word);
      }
      //početak produkcije, završni znak lijeve strane?
      if(word[0] == '<' && word[word.length-1]=='>')
      {
        debug writefln("--------------POCETAK PRODUKCIJE");
        //nezavrsni pronađen
        if(((word in nonTerminalToInt) is null))
        {
          nonTerminalToInt[word.dup] = nonTerminalToInt.length;
          debug writefln("nezavrsni: ", word, " = ", nonTerminalToInt[word]);
        }
        
        left = nonTerminalToInt[word];

        if(productionNO==0)
        {
          grammarFirstNonTerminal = left;
          productionNO++;
        }
        file.readf("%s",&word);
        debug writef("%s ",word);
        
        //nakon lijeve strane ne ide -> znači da su to arguments
        if(word != "->")
        {
          arguments = null;
          do
          {
            debug writef("%s ",word);
            arguments ~= word ~ " ";
          }
          while(file.readf("%s",&word), word != "->")
          debug writefln("arguments are: ",arguments);
        }
      }
      
      if(word != "->")
      {
        file.readf("%s",&word);
        debug writef("%s ",word);
        if(word != "->"){throw new GrammarException("Greska nedostaje -> nakon lijeve strane produkcije\n");}
      }
        
		  file.readf("%s",&word);
		  debug writef("%s ",word);
      if(word != "{{"){throw new GrammarException("Greska nedostaje {{ nakon '->' !\n");}
      
      //ovdje počinje produkcija i traje sve do }}
      dijeloviProdukcije = null;
      while(file.readf("%s",&word), word != "}}")
      {
        debug writef("%s ",word);
        dijeloviProdukcije ~= word.dup;
        
        //unutar produkcije nezavršni znak?
        if(word[0] == '<' && word[word.length-1]=='>')
        {
          //nezavrsni pronađen
          if(((word in nonTerminalToInt) is null))
          {
            nonTerminalToInt[word.dup] = nonTerminalToInt.length;
            debug writefln("nezavrsni: ", word, " = ", nonTerminalToInt[word]);
          }
          right ~= nonTerminalToInt[word];
        }
        else
        //unutar produkcije završni znak?
        if(word[0] == '"' && word[word.length-1]=='"')
        {
          //terminal found
          word = word[1..word.length-1]; 
          if(((word in terminalToInt) is null))
          {
            terminalToInt[word.dup] = -(terminalToInt.length+1);
            debug writefln("zavrsni  : ", word, " = ", terminalToInt[word]);
          }
          right ~= terminalToInt[word];
        }
        else
        {
          //inače je odsječak koda:
        }
      }
      debug writefln();
      
      previousRule = new GrammarRule(left,right);
      debug writefln("RULE:: %s",previousRule.toStringNumbers(this));
      grammarRules.insert(previousRule);
      debug writefln("--------------KRAJ PRODUKCIJE");
      
      
      //sad treba alocirati mjesto za grc dijelove i pridjeliti ih.
      grc.parts.length = previousRule.right.length+1;
      grc.calls.length = previousRule.right.length;
      if(arguments.length > 0)
      {
        grc.arguments = arguments[1..arguments.length-2].dup;
      }
      int partNumber = 0;
      debug writefln("Produkcija ima ",dijeloviProdukcije.length," dijelova");
      foreach(int i, string part; dijeloviProdukcije)
      {
        debug writefln("current part is: ", partNumber);
        if( (part[0] == '<' && part[part.length-1]=='>')
          ||(part[0] == '"' && part[part.length-1]=='"'))
        {
          partNumber++;
        }
        else if(part.length>=5 && part[0..5]=="call(")
        {
          grc.calls[partNumber-1] = part[5..part.length-2].dup;
        }
        else
        {
          grc.parts[partNumber] ~= part.dup ~ " ";
        }
        debug writefln("Parsiram kod iz ",i," %s",part);
      }

      rule2Code[previousRule] = grc;
      grc = new GrammarRuleCode;
      right.length = 0;
      
      /*
        counter = 0;
				int[]  right;
				if(splitLine.length>=1 && splitLine[0].length>=2 && splitLine[0][0..2]=="//")
				{
					continue;
				}
				foreach(int index,char[] part; splitLine)
				{
					version(GrammarTest)printf("\tpart: %.*s\n",part);
					if(part == "->")continue;
					if(part[0] == '<' && part[part.length-1]=='>' && part != "<>")
					{
						//nonterminal found
						version(GrammarTest)printf(" nezavrsni\n");

						if(((part in nonTerminalToInt) is null))
						{
							version(GrammarTest)printf(" novi");
							nonTerminalToInt[part] = nonTerminalToInt.length;
							version(GrammarTest)printf(" %d\n",nonTerminalToInt[part]);
						}

						if(counter == 0)
						{
							left = nonTerminalToInt[part];

							if(productionNO==0)
							{
								grammarFirstNonTerminal = left;
								productionNO++;
							}
						}
						else
						{
							right ~= nonTerminalToInt[part];
						}
						counter++;
					}
					else
					{
						//terminal found
						if(part != "$")
						{
							version(GrammarTest)printf(" zavrsni\n");

							if(((part in terminalToInt) is null))
							{
								version(GrammarTest)printf(" novi");
								terminalToInt[part] = -(terminalToInt.length+1);
								version(GrammarTest)printf(" %d",terminalToInt[part]);
							}

							right ~= terminalToInt[part];
						}
					}
				}

				if(splitLine.length >= 2)
				{
					previousRule = new GrammarRule(left,right);
					debug writefln("RULE:: %s",previousRule.toStringNumbers(this));
					grammarRules.insert(previousRule);
				}
			}
		*/
		}
		while(!file.eof);

		debug writefln("parsing file done");

		//printf("\nUcitavanje datoteke trajalo: %dms",clock()-pocetak);

		terminalToInt["EOF"] = Special.EOF;
		terminalToInt["@"] = Special.TOCKA;


		int[] dstr0; dstr0~=grammarFirstNonTerminal; // int[] dstr = [grammarFirstNonTerminal];

		grammarRules.insert(new GrammarRule(parsersFirstNonTerminal,dstr0));


		//it is possible to print GrammarRules only after these two initialization 
		foreach(char[] nez, int num; nonTerminalToInt)
		{
			nonTerminalToString[num] = nez;
		}
		foreach(char[] zav, int num; terminalToInt)
		{
			terminalToString[num] = zav;
		}


		nonTerminalToInt.rehash;
		nonTerminalToString.rehash;

		terminalToInt.rehash;
		terminalToString.rehash;
		
		this.calculateEPSILON();
		this.calculateBEGINSWITH();
		this.calculateFOLLOW();

		debug this.printGrammarRules();		
		debug IspisiNezavrsne();
		debug printTerminals();
			
		debug writefln("finished loading grammar file!");			
	}
	

  /+
	GrammarRuleCode ParseGrammarRuleCode(File file, char[][]splitLine,GrammarRule previous)
	{
		GrammarRuleCode grc = new GrammarRuleCode;
		
		foreach(char[] part; splitLine)
		{
			debug printf("\n#%.*s#",part);
			grc.arguments ~= (strip(part) ~ " ");
		}
		//debug printf("\n%d arguments #%.*s#",previous.right.length,grc.arguments);
		char[] line;
		char[][] split2;
		
		grc.parts.length = previous.right.length+1;
		grc.calls.length = previous.right.length;
		int currentPart = 0;
		
		do
		{
			line = file.readLine();
			//debug printf("\t%.*s\n",line);
			split2 = split(line);
			if(split2.length>=1 && split2[0]=="}}}")
			{
				break;
			}
			else
			if(split2.length>=2 && split2[0]=="part")
			{
				currentPart = std.conv.toInt(split2[1]);
				//debug printf("\ncurrent part from now is %d\n",currentPart);
			}
			else
			if(split2.length>=2 && split2[0]=="call")
			{
				for(int i=1; i<split2.length; i++)
				{
					grc.calls[currentPart] ~= strip(split2[i]);
				}
			}
			else
			{
				grc.parts[currentPart] ~= strip(line);
			}
		}while(true);
		/*debug foreach(int index,char[][] part;grc.parts)
		{
			printf("\nPart %d",index);
			foreach(char[] line; part)
			{
				printf("\n\t%.*s",line);
			}
			if(index<grc.calls.length)printf("\n\tcall %.*s",grc.calls[index]);
		}*/
		return grc;
	}
	
	+/
  public void IspisiNezavrsne()
	{
		printf("\nNezavrsni: pocetni: %.*s\n",nonTerminalToString[grammarFirstNonTerminal]);
		foreach(char[] nez, int num; nonTerminalToInt)
		{
			printf("%.*s=>%d\n",nez,num);
		}
	}

	public int getIndex(char[] znak)
	{
		if(znak in nonTerminalToInt)return nonTerminalToInt[znak];
		if(znak in terminalToInt)return terminalToInt[znak];
		return 0; //has to return 0
	}

	public char[] getZnak(int index)
	{
		if(index in nonTerminalToString)return nonTerminalToString[index];
		if(index in terminalToString)return terminalToString[index];
		return null;
	}

	private void calculateFOLLOW()
	{
		clock_t pocetak = clock();
		version(GrammarTest)printf("\ncalculateFOLLOW");
		foreach(int zn; nonTerminalToInt)
		{
			FOLLOW[zn] = new IntegerSet();
		}

		FOLLOW[nonTerminalToInt["<S_>"]].insert(Special.EOF);

		int size;
		bool change;
		int j;

		do
		{
			change = false;
			foreach(inout int zn; nonTerminalToInt)
			{
				foreach(GrammarRule prod; grammarRules.Values())
				{
					foreach(int index, int i; prod.right)
					{
						if(i==zn)
						{
							if(i==prod.right[prod.right.length-1])//da li je i==zadnji znak iz grammarRules
							{
								size=FOLLOW[zn].length;
								FOLLOW[zn].insert(FOLLOW[prod.left]);
								if(size != FOLLOW[zn].length) change|= true;
								break;
							}

							j = index+1; //j index on the right side
							for(; j<prod.right.length; j++)
							{
								if(j!=prod.right.length)
								{
									if(prod.right[j]<0) //if terminal on j(th) place
									{
										size = FOLLOW[zn].length;
										FOLLOW[zn].insert(prod.right[j]);
										if(size != FOLLOW[zn].length)change|= true;
										break;
									}
									else
									{
										foreach(GrammarRule p2; grammarRules.Values())
										{
											if(p2.left == prod.right[j])
											{
												size = FOLLOW[zn].length;
												FOLLOW[zn].insert(BEGINSWITH[p2]);
												if(size != FOLLOW[zn].length)change |= true;
											}
										}
										if(!epsilonZnakovi.contains(prod.right[j]))break;
									}
								}
							}
						}
					}
				}
			}
		}
		while(change);

		//printf("\nRacunanje FOLLOW trajalo: %dms",clock()-pocetak);

		version(GrammarTest)
		{
			foreach(int index, IntegerSet iset; FOLLOW)
			{
				printf("\niza %.*s FOLLOW: ",nonTerminalToString[index]);
				foreach(int x; iset.Values())
				{
					printf("%.*s ",terminalToString[x]);
				}
			}
		}
	}

	private void calculateBEGINSWITH()
	{
		clock_t pocetak = clock();
		version(GrammarTest)printf("\ncalculateBEGINSWITH");
		debug writefln("calculateBEGINSWITH");
		foreach(int zn; nonTerminalToInt)
		{
			BEGINSWITHZNAK[zn] = new IntegerSet();
		}

		foreach(GrammarRule p; grammarRules.Values())
		{
			BEGINSWITH[p] = new IntegerSet();
		}
		bool promjena;

		do
		{
			promjena = false;
			foreach(GrammarRule prod; grammarRules.Values())
			{
				if(prod.right.length > 0)
				{
					foreach(int index, int zn; prod.right)
					{
						if(zn < 0)//zavrsni
						{
							promjena |= BEGINSWITH[prod].insert(zn);
							break;
						}
						else if(zn >= 0)
						{
							promjena |= DodajBEGINSWITHZnaka(prod,zn);
							if(!epsilonZnakovi.contains(zn))break;
						}
					}
				}
			}
		}
		while(promjena);
		
		foreach(GrammarRule prod ;grammarRules.Values())
		{
			BEGINSWITHZNAK[prod.left].insert(BEGINSWITH[prod]);
		}

		//printf("\nRacunanje BEGINSWITH trajalo: %dms",clock()-pocetak);

		version(GrammarTest)
		foreach(GrammarRule p, IntegerSet s; BEGINSWITH)
		{
			printf("\nZAP: %.*s\n\t",p.toString(this));
			foreach(int i; s.Values())
			{
				printf("%.*s ",terminalToString[i]);
			}
		}
	}

	private bool calculateEPSILON()
	{
		clock_t pocetak = clock();
		version(GrammarTest)printf("\ncalculateEPSILON");
		epsilonZnakovi = new IntegerSet();

		foreach(inout GrammarRule p; grammarRules.Values())
		{
			if(p.isEpsilon)epsilonZnakovi.insert(p.left);
		}

		int length;

		do
		{
			length = epsilonZnakovi.length;
			foreach(GrammarRule i; grammarRules.Values())
			{
				if(i.SviNezavrsni())
				{
					bool svieps=true;
					foreach(inout int elem; i.right)
					{
						if(!epsilonZnakovi.contains(elem))
						{
							svieps=false;
							break;
						}
					}

					if(svieps)
					{
						epsilonZnakovi.insert(i.left);
					}
				}
			}
		}
		while(length != epsilonZnakovi.length);

		//printf("\nRacunanje EPSILON trajalo: %dms",clock()-pocetak);

		version(GrammarTest)//ispis
		{
			printf("\n\tEpsilon znakova je %d i to su:",epsilonZnakovi.length);
			foreach(int x; epsilonZnakovi.Values())
			{
				printf("%.*s ",nonTerminalToString[x]);
			}
		}
		return true;
	}

	private bool DodajBEGINSWITHZnaka(GrammarRule prod, int zn)
	{
		bool promjena=false;
		int size;
		foreach(GrammarRule prod2; grammarRules.Values())
		{
			if(prod2.left == zn)
			{
				size = BEGINSWITH[prod].length;
				BEGINSWITH[prod].insert(BEGINSWITH[prod2]);

				if(size != BEGINSWITH[prod].length)
					promjena = true;
			}
		}
		return promjena;
	}

	public void printTerminals()
	{
		printf("\n\nZavrsni:\n");
		foreach(char[] zav, int num; terminalToInt)
		{
			printf("%.*s=>%d\n",zav,num);
		}
	}

	void printGrammarRules()
	{
    writefln("Main code:\n",mainCode);
		writefln("grammarRulesCount: %d",grammarRules.length);
		foreach(GrammarRule p; grammarRules.Values())
		{
			writef("\n", epsilonZnakovi.contains(p.left), "\t", p.toString(this));
			if(p in rule2Code)
			{
				GrammarRuleCode grc = rule2Code[p];
				foreach(int index,char[][] part;grc.parts)
				{
					printf("\nPart %d",index);
					foreach(char[] line; part)
					{
						writef("%s ",line);
					}
				}
        writefln();
        foreach(int index, char[] callsList; grc.calls)
        {
          writefln("calls with: ", grc.calls[index]);
        }
			}
		}
	}

	GrammarRule[] getProductionsByLeftSide(int leftside)
	{
		GrammarRule[] prods;
		foreach(GrammarRule p; grammarRules.Values())
		{
			if(p.left == leftside)
			{
				prods ~= p;
			}
		}
		return prods;
	}

	package GrammarRuleCode[GrammarRule] rule2Code;
	package char[] mainCode;
	private IntegerSet[int] FOLLOW;
	private IntegerSet[int] BEGINSWITHZNAK;
	private IntegerSet[GrammarRule] BEGINSWITH;
	public int[char[]] nonTerminalToInt;
	public int[char[]] terminalToInt;
	private char[][int] nonTerminalToString;
	private char[][int] terminalToString;
	private IntegerSet epsilonZnakovi;
	private int grammarFirstNonTerminal;
	private int parsersFirstNonTerminal;
	private GrammarRuleSet grammarRules;
	private Setup GlobalSetup;
}

/*
	Stavka is part of the State (but i don't know how to say it in English)
	Stavka is grammar rule plus it's follow set
*/
private class Stavka
{
	this(Grammar G,GrammarRule p, int look)
	{
		prod = p;
		lookahead = new IntegerSet();
		lookahead.insert(look);
		this.G = G;
	}

	this(Grammar G,GrammarRule p)
	{
		prod = p;
		lookahead = new IntegerSet();
		this.G = G;
	}

	char[] toString()
	{
		char[] str;
		str ~= prod.toString(G) ~ " {";
		foreach(int i; lookahead.Values())
		{
			str ~= G.terminalToString[i] ~ " ";
		}

		str ~= " }";
		return str;
	}

	version(enableHTMLoutput)
	{
		void toHTML(Stream file)
		{
			file.printf("<td nowrap>");
			this.prod.toHTML(file,G);
			file.printf("</td>");
			version(none)
			{
				foreach(int look; lookahead.Values())
				{
					file.printf("<td>%.*s</td>",G.terminalToString[look]);
				}
			}
		}
	}

	static bool DodajStavku(Stavka stavka, StavkaSet ss)
	{
		foreach(inout Stavka stav; ss.Values())
		{
			if(stav.prod == stavka.prod)
			{
				int size=stav.lookahead.length;
				stav.lookahead.insert(stavka.lookahead);

				if(size!=stav.lookahead.length)
					return true;
				else return false;
			}
		}
		ss.insert(stavka);
		return false;
	}

	int opEquals(Object o)
	{
		Stavka s2 = cast(Stavka)o;
		//maybe not the smartest idea to have a context sensitive opEquals
		//but it was the easiest way at the moment i was adding lalr option
		//-should be changed
		if(this.G.GlobalSetup.lalr==true)
		{
			if(this.prod == s2.prod)
			{
				this.lookahead.insert(s2.lookahead);
				return true;
			}
			else return false;
		}
		else
		{
			if(this.prod == s2.prod && this.lookahead == s2.lookahead)
				return true;
			else return false;
		}
	}

	uint toHash()
	{
		int broj = prod.toHash;
		foreach(int i; lookahead.Values())
		broj += i;
		return broj;
	}

	int opCmp (Object o)  { Stavka x = cast(Stavka)o; return this.toHash < x.toHash; }

	private Grammar G;
	private GrammarRule prod;
	private IntegerSet lookahead;
}

private class Transititon
{
	this(int zn, int st)
	{
		znak = zn; brstanja = st;
	}

	char[] toString()
	{
		char[] result;
		if(this.znak < 0)
		{
			result ~= G.terminalToString[this.znak];
		}
		else
		{
			result ~= G.nonTerminalToString[this.znak][1..length-1];
		}
		result ~= "=>";
		result ~= IntToString(this.brstanja);
		return result;
	}
	int opEquals(Transititon b)
	{
		if(this.znak == b.znak && this.brstanja == b.brstanja)
		{
			return true;
		}
		else return false;
	}
	static Grammar G;
	int znak, brstanja;
}

private class State
{
	this()
	{
		stavke = new StavkaSet();
		BR = StateCounter ++;
	}

	char[] toString()
	{
		char[] str;
		str ~= "State: ";
		str ~= IntToString(BR);
		foreach(Stavka st; stavke.Values())
		{
			str ~= "\n" ~ st.toString();
		}

		str ~= "\n---";
		foreach(Transititon pr; pri)
		{
			str ~= "\n" ~ pr.toString();
		}
		return str;
	}

	bool ProsiriState()
	{
		bool promjena;
		int size;
		//debug printf("State.ProsiriState\n");
		do
		{
			promjena = false;
			size = this.stavke.length;
		
			foreach(Stavka i; this.stavke.Values())
			{
				//debug printf(" trenutna stavka: %.*s polozaj tocke %d\n" , i.toString, i.prod.PolozajTocke);
				if(i.prod.right.length>=1)//ako su bar dva znaka...
				{
					if(!(i.prod.right.length==i.prod.PolozajTocke))//nije na zadnjem mjestu tocka
					{
						if(i.prod.nonTerminalAfterDot())
						{
							int polozajtocke = i.prod.PolozajTocke;
							if((polozajtocke)>(i.prod.right.length))break;

							promjena = DodajKojeZapocinjuS(G,this,i.prod.right[polozajtocke],i,polozajtocke);

							if(size != this.stavke.length)promjena|=true;
						}

					}

				}
			}

		}
		while(promjena)

		return true;
	}

	State IzracunajStateZaZnak(int znak)
	{
		int polozajtocke;
		bool imaprod = false;
		State s = new State();
		foreach(Stavka stavka; this.stavke.Values())
		{
			if(!stavka.prod.isEpsilon)
			{
				if(!(stavka.prod.right.length==stavka.prod.PolozajTocke))
				{
					if(stavka.prod.ZnakIzaTocke() == znak)
					{
						foreach(GrammarRule g; G.grammarRules.Values())
						{
							if(GrammarRule.Cmp(stavka.prod,g))
							{
								polozajtocke = stavka.prod.PolozajTocke();
								GrammarRule p = GrammarRule.newGrammarRuleWithDot(g,polozajtocke+1);

								if(!(p is null))
								{
									Stavka novastavka = new Stavka(G,p);
									novastavka.lookahead.insert(stavka.lookahead);
									s.DodajStavku(novastavka);
									imaprod = true;
								}
							}
						}
					}
				}
			}
		}

		if(imaprod)
		{
			s.ProsiriState();
			return s;
		}
		else
		{
			State.StateCounter--; return null;
		}
	}

	static bool DodajKojeZapocinjuS(Grammar G,StavkaSet ss,int znak,Stavka stavka0, int pos)
	in{assert(znak>=0);//znak je nezavrsni
	  }
	body{
		GrammarRule stockom;
		int pos_2 = pos, size;
		bool promjena=false;;
		foreach(GrammarRule i; G.grammarRules.Values())
		{
			pos = pos_2;
			size = ss.length;

			if(i.left == znak)
			{
				stockom = GrammarRule.newGrammarRuleWithDot(i,0);
				Stavka stavka = new Stavka(G,stockom);

				if(pos == stavka0.prod.right.length-1)
				{//ako je znak iza tocke u produkciji koja se proÄśô ĄËśôóˇiruje zadnji znak
					stavka.lookahead.insert(stavka0.lookahead);
				}

				else
				{
					pos++;

					while(pos < stavka0.prod.right.length)
					{
						if(stavka0.prod.right[pos]<0)
						{//zavrsni
							stavka.lookahead.insert(stavka0.prod.right[pos]);
							break;
						}
						else
						{//nezavrsni

							if(pos == stavka0.prod.right.length-1)
							{
								if(G.epsilonZnakovi.contains(stavka0.prod.right[pos]))
									stavka.lookahead.insert(stavka0.lookahead);
							}

							stavka.lookahead.insert(G.BEGINSWITHZNAK[stavka0.prod.right[pos]]);

							if(!G.epsilonZnakovi.contains(stavka0.prod.right[pos]))
							{
								break;
							}
						}
						pos++;
					}
				}
				promjena=Stavka.DodajStavku(stavka,ss);
			}
		}
		return false;
	}

	static bool DodajKojeZapocinjuS(Grammar G,State s,int znak,Stavka stavka0, int pos)
	{
		return DodajKojeZapocinjuS(G,s.stavke,znak,stavka0,pos);
	}

	public void ZnakoviIzaTocke(inout int[] znakovi)
	{
		znakovi.length = 0;
		foreach(Stavka i; this.stavke.Values())
		{
			if(!i.prod.isEpsilon() && (i.prod.PolozajTocke() < i.prod.right.length))
			{
				znakovi ~= i.prod.right[i.prod.PolozajTocke];
			}
		}
	}

	bool DodajStavku(Stavka stavka)
	{
		return Stavka.DodajStavku(stavka,this.stavke);
	}

	int opEquals (Object o)
	{
		State x=cast(State)o;
		bool ret;

		if(this.stavke == x.stavke)
			ret = true;
		else ret = false;
		
		return ret;
	}

	version(enableHTMLoutput)
	{
		void toHTML(Stream file)
		{
			file.printf("<table border><tr><td nowrap>State - %d</td>",this.BR);
			file.printf("<td><table border>");
			foreach(Stavka st; this.stavke.Values())
			{
				file.printf("<tr><td nowrap>");
				st.toHTML(file);
				file.printf("</td></tr>");
			}
			char[] str;
			file.printf("</br>BR.TRANS:%d",this.pri.length);
			foreach(int i,Transititon trans; this.pri)
			{
				file.printf("<tr><td nowrap>T%d-",i);
				str = trans.toString();
				file.printf(str);
				file.printf("</td></tr>");
			}
			file.printf("</table></td>");
		}
	}
		
	uint toHash()
	{
		int broj = 0;
		foreach(Stavka s; stavke.Values())
		broj += s.toHash;
		return broj;
	}

	int opCmp (Object o)
	{
		State drugo = cast(State)o;
		return this.toHash < drugo.toHash;
	}

	int BrojNeREDUCTION()
	{
		int broj = 0;
		foreach(Stavka s; stavke.Values())
		{
			if(s.prod.right.length != s.prod.PolozajTocke)
				broj++;
		}
		return broj;
	}

	static Grammar G;
	static int StateCounter = 0;
	bool obradjeno = false;
	private int BR;
	private StavkaSet stavke;
	private Transititon[] pri;
}

private class RowElement
{
	this(int i1, int i2,Grammar g)
	{
		prvi = i1; drugi = i2;
		G = g;
	}

	this(int i1, int i2, int zn,Grammar g)
	{
		prvi = i1; drugi = i2; znak = zn;
		G = g;
	}

	this(Stream file)
	{
		file.read(prvi);

		if(this.prvi == Special.REDUCTION)
		{
			file.read(drugi);
			file.read(znak);
		}
	}

	char[] toString()
	{
		char[] rezult;

		if(this.prvi >= 0)
		{
			rezult ~= IntToString(this.prvi); return rezult;
		}

		else
			if(this.prvi == Special.REDUCTION)
			{
				rezult ~= "R" ~ IntToString(this.drugi) ~ G.nonTerminalToString[this.znak];
				return rezult;
			}

			else
				if(this.prvi == Special.ACCEPT)
				{
					rezult = "PRI"; return rezult;
				}

				else
				{
					return "";
				}
	}

	version(enableHTMLoutput)
	{
		void toHTML(Stream file)
		{
			if(this.prvi >= 0)
			{
				file.printf("%.*s",IntToString(this.prvi));
			}
	
			else
				if(this.prvi == Special.REDUCTION)
				{
					file.printf("R%.*s&lt%.*s&gt",IntToString(this.drugi),G.nonTerminalToString[this.znak][1..G.nonTerminalToString[this.znak].length-1]);
				}
	
				else
					if(this.prvi == Special.ACCEPT)
					{
						file.printf("PRI");
					}
	
					else
					{}
	
		}
	}

	int opEquals (RowElement x)
	{
		if(this.prvi == x.prvi && this.drugi == x.drugi && this.znak == x.znak)return true;
		else return false;
	}

	bool toStream(Stream file)
	{
		if(this.prvi == Special.REDUCTION)
		{
			file.write(prvi);
			file.write(drugi);
			file.write(znak);
		}

		else
		{
			file.write(prvi);
		}

		return true;
	}

	private Grammar G;
	int prvi,drugi,znak;
}

private class ParserDevice
{
	this(Parser p, int st, int ulznak)
	{
		this.P = p;
		State = st;
		trenutnizn = ulznak;
		myID = pdID++;
		finished = false;
		accepted = false;
		stog ~= 0;
	}

	this(Parser p, int[] stack, int st, int ulznak)
	{
		this.P = p;
		State = st;
		trenutnizn = ulznak;
		myID = pdID++;
		finished = false;
		accepted = false;
		stog = stack.dup;
	}

	this(ParserDevice p2)
	{
		this.P = p2.P;
		State = p2.State;
		trenutnizn = p2.trenutnizn;
		myID = p2.pdID;
		finished = p2.finished;
		accepted = p2.accepted;
		stog = p2.stog.dup;
	}

	ParserDevice dup()
	{
		return new ParserDevice(this);
	}
/*
	void Print()
	{
		if(P.GlobalSetup.parserOutput)
		{
			printf("\nP: (%2d,%3d,%3d) -> ",this.myID,this.State,this.trenutnizn);
			foreach(int I; stog)
			{
				printf("%d,",I);
			}

			printf(" :: ");

			for(int i=trenutnizn; i<P.ulaz.length; i++)
			{
				printf("%.*s ",P.ulaz[i]);
			}
		}
	}
*/
	void PrintI()
	{
		if(P.GlobalSetup.parserOutput)
		{
			printf("\rPi: (%2d,%3d,%3d) -> ",this.myID,this.State,this.trenutnizn);
			
			printf("%d",stog.length);
			/*foreach(int I; stog)
			{
				printf("%d,",I);
			}*/

			printf(" :: ");
			
			
			for(int i=trenutnizn; i<trenutnizn+15 && i<P.ulazI.length; i++)
			{
				printf("%.*s ",this.P.G.terminalToString[P.ulazI[i].getIndex]);
			}
		}
	}

	void ParseTreeToXML(Stream file)
	{
		file.printf("<?xml version=\"1.0\"?>");
		foreach(int i,ILENode n; nodesI)
		{
			PrintILENode2XML(n,file);
		}
	}

	void PrintILENode2XML(ILENode a,Stream file)
	{
		int elemIndex = a.element.getIndex;
		if(elemIndex < 0)
		{
			file.printf("<_ _=\"%.*s\">\n",P.G.terminalToString[elemIndex]);
		}
		else
		{
			file.printf("<_ _=\"%.*s\">\n",
				P.G.nonTerminalToString[elemIndex][1..P.G.nonTerminalToString[elemIndex].length-1]);
		}

		if(a.subNodes.length != 0)
		{
			foreach(int i,ILENode xx; a.subNodes)
			{
				PrintILENode2XML(xx,file);
			}
		}

		if(elemIndex < 0)
		{
			file.printf("</_>",P.G.terminalToString[elemIndex]);
		}
		else
		{
			file.printf("</_>",P.G.nonTerminalToString[elemIndex][1..P.G.nonTerminalToString[elemIndex].length-1]);
		}
	}

/*	void PrintIntNode(IntNode a, int level)
	{
		if(a.element < 0)
			printf("%.*s ",P.G.terminalToString[a.element]);
		else
			printf("%.*s ",P.G.nonTerminalToString[a.element]);

		if(a.subNodes.length != 0)
		{
			printf(" ( ");

			foreach(int i,IntNode xx; a.subNodes)
			{
				PrintIntNode(xx,level+1);
			}

			printf(" ) ");
		}
	}*/

	bool OneStepI(int stepID)
	{
		static int outputNo = 0;

		if(trenutnizn >= P.ulazI.length)
		{
			finished= true;
			return false;
		}

		if(!(P.ulazI[trenutnizn].getIndex in P.G.terminalToString))
		{
			char[] err = "\nPogresan znak ulaza " ~ IntToString(P.ulazI[trenutnizn].getIndex);
			throw new Parser.ParsingException(err);
		}

		int pom = P.znakToIndex[P.ulazI[trenutnizn].getIndex];
		debug writefln("OneStepI::(%d,%d[%d])=%s",State,pom,P.ulazI[trenutnizn].getIndex,P.tablica[State][pom] is null? 0 :1);
		if(P.tablica[State][pom] is null)
		{
			if(P.GlobalSetup.parserOutput)printf(" NO ACTION(null row)!");
			finished = true;
			return false;
		}
		else
		{
			foreach(RowElement er; P.tablica[State][pom].Values())
			{
				if(er.prvi == Special.ACCEPT && trenutnizn==P.ulazI.length-1)
				{
					if(P.GlobalSetup.parserOutput)printf("\nACCEPT!");

					this.accepted=true;

					if(P.GlobalSetup.printParseTree2XML)
					{
						File file = new File();
						file.create(P.GlobalSetup.inputfile~IntToString(outputNo++)~".tree.xml",FileMode.Out);
						ParseTreeToXML(file);
						file.close();
					}

					finished = true;
					return false;
				}
			}
			
			if(P.tablica[State][pom][stepID].prvi>=0)
			{
				if(P.GlobalSetup.parserOutput)printf(" -> POMAKNI");
				nodesI ~= new ILENode(P.ulazI[trenutnizn]);
				trenutnizn++;
				State = P.tablica[State][pom][stepID].prvi;
				stog ~= State;
				return true;
			}

			else
				if(P.tablica[State][pom][stepID].prvi == Special.REDUCTION)
				{
					if(P.GlobalSetup.parserOutput)printf(" -> REDUCIRAJ %d, %.*s",
															 P.tablica[State][pom][stepID].drugi,
															 P.G.nonTerminalToString[P.tablica[State][pom][stepID].znak]
															);

					if(P.tablica[State][pom][stepID].drugi==0)
					{
						int znakic = P.tablica[State][pom][stepID].znak;
						nodesI ~=
						new ILENode(
							P.ulazI[0].create(
								P.G.nonTerminalToString[znakic]
							)
						);
					}

					else
					{
						int from = nodesI.length-P.tablica[State][pom][stepID].drugi;
						int to = nodesI.length;

						if(from>=0 && from < nodesI.length)
						{
							ILENode nova =
							new ILENode(
								P.ulazI[0].create(
									P.G.nonTerminalToString[P.tablica[State][pom][stepID].znak]
								),
								nodesI[from..to].dup
							);
							nodesI.length = nodesI.length - P.tablica[State][pom][stepID].drugi;
							nodesI ~= nova;
						}
					}

					int pom2 = P.znakToIndex[P.tablica[State][pom][stepID].znak];
					stog.length = stog.length - P.tablica[State][pom][stepID].drugi;
					State = stog[stog.length-1];
					State = P.tablica[State][pom2][0].prvi;
					stog ~= State;
					return true;
				}
		}

		return false;
	}

	bool BranchI(inout ParserDevice[] parseri)
	{
		if(!(P.ulazI[trenutnizn].getIndex in P.G.terminalToString))
		{
			char[] err = "\nWrong input (while branching): " ~ P.G.getZnak(P.ulazI[trenutnizn].getIndex);
			throw new Parser.ParsingException(err);
		}

		int pom = P.znakToIndex[P.ulazI[trenutnizn].getIndex];

		debug writefln("BranchI::stupac(pom)=%s",pom);

		if(P.tablica[State][pom] !is null)
		{
			if(P.tablica[State][pom].length > 1)
			{
				foreach(RowElement r; P.tablica[State][pom].Values())
				{
					if(r.prvi == Special.ACCEPT)
					{
						return true;
						printf("ACCEPT!\n");
					}
				}

				if(P.GlobalSetup.parserOutput)
					printf("\nIma vise Transititona %d i to %d",this.myID,P.tablica[State][pom].length);

				for(int i=0; i<P.tablica[State][pom].length-1; i++)
				{
					parseri ~= new ParserDevice(this.P,stog,this.State,this.trenutnizn);
					parseri[parseri.length-1].nodesI = this.nodesI.dup;
				}
			}
		}
		else
		{
			if(P.GlobalSetup.parserOutput)writefln("Null row element at [", State, "][", pom, "]");
		}
		return true;
	}

	int opEquals (ParserDevice x)
	{
		if(this.State != x.State)return false;
		if(this.trenutnizn != x.trenutnizn)return false;
		if(this.stog.length != x.stog.length)return false;
		if(this.stog[] != x.stog[])return false;
		return true;
	}
	ILENode tree()
	{
		if(nodesI is null)throw new Parser.ParsingException("No tree available!");
		return nodesI[0];
	}
	

	Parser P;
	public ILENode[] nodesI;
	private bool finished = false;
	private int State,trenutnizn;
	private bool accepted = false;
	private int[] stog;
	private int myID;
	static int pdID = 0;;
}

public class Parser
{
	static class ParsingException:Exception
	{
		this(char[]msg)
		{
			super(msg);
		}
	}
	
	public enum Set:ubyte{Input,ParserOutput,Tree2XML,Recalc,Lalr,LalrExtra};
	public enum SplitType:ubyte{SpaceSplit,CharByCharSplit};
	
	public this(...)
	{
		this.GlobalSetup = new Setup();
		
		for (int i = 0; i < _arguments.length; i++)
		{	
			if (_arguments[i] == typeid(Set))
			{
				//int j = std.stdarg.va_arg!(ubyte)(_argptr);
				int j = *cast(int *)_argptr;
				//int j = *_argptr.cast(int*);
				_argptr += int.sizeof;

				switch(j)
				{
					case Set.Input:
					//case Set.Test:
					{
						i++;
						if(_arguments[i] == typeid(char[]))
						{
							//char[] str = std.stdarg.va_arg!(char[])(_argptr);
							char[] str = (*cast(char[] *)_argptr).dup;
							_argptr += (char[]).sizeof;
							switch(j)
							{
								case Set.Input: 
									this.GlobalSetup.inputfile=str;
									break;
								/*case Set.Test:
									this.GlobalSetup.testfile=str;
									break;*/
							}
						}
						else throw new ParsingException("Wrong argument, expected char[]");
						break;
					}
					case Set.ParserOutput:
						this.GlobalSetup.parserOutput=true;
						break;
					case Set.Tree2XML:
						this.GlobalSetup.printParseTree2XML=true;
						break;
					case Set.Recalc:
						this.GlobalSetup.forseRecalc=true;
						break;
					case Set.Lalr:
						this.GlobalSetup.lalr=true;
						break;
					case Set.LalrExtra:
					{
						i++;
						if(_arguments[i] == typeid(int))
						{
							int passes = (*cast(int *)_argptr);
							_argptr += (int).sizeof;
							this.GlobalSetup.lalrExtraPasses = passes;
						}
						else throw new ParsingException("Wrong argument, expected int (number of extra passes)");
						break;
					}					
						this.GlobalSetup.lalr=true;
						break;
					
					default:
						throw new ParsingException("Wrong argument");
				}
			}
		}
		this(this.GlobalSetup);
	}
	private this(Setup s)
	{
		this(new Grammar(s),s);
	}
	private this(Grammar gram,Setup s)
	{
		GlobalSetup = s;
		Stanja = new StateSet();
		proturjecnosti = new CharArraySet();
		G = gram;
		this.FillZnakToIndex();
		bool savetable = false;

		if(!GlobalSetup.forseRecalc)
		{
			try
			{
				version(zippedTableFile)
				{
					if(getSize(GlobalSetup.inputfile~".zLRtbl")!=0)
					{
						auto File ftable = new File(GlobalSetup.inputfile~".zLRtbl",FileMode.In);
						void[] uncompressed = std.zlib.uncompress(ftable.toString,ftable.size*20);
						this.tableFromStream(new MemoryStream(cast(ubyte[])uncompressed));
						ftable.close();
						return;
					}
				}
				else
				{
					if(getSize(GlobalSetup.inputfile~".LRtbl")!=0)
					{
						auto File ftable = new File(GlobalSetup.inputfile~".LRtbl",FileMode.In);
						this.tableFromStream(ftable);
						return;
					}
				}
			}
			catch(Exception e)
			{
				printf("\nParser::this::Exception wile reading: %.*s",e.msg);
				savetable = true;
			}
		}
		State.StateCounter=0;
		this.IzracunajStanja();
		this.KonstruirajTablicu();
		version(enableHTMLoutput)this.TablicaToFile();
		if(GlobalSetup.forseRecalc || savetable)
		{
			try
			{
				version(zippedTableFile)
				{
					MemoryStream m = new MemoryStream();
					auto File fileZipped = new File;
					if(fileZipped is null)
					{
						printf("\nParser::this::error creating table file!"); 			
					}
					fileZipped.create(GlobalSetup.inputfile~".zLRtbl",FileMode.Out);
					this.toStream(m);
					writefln("creating memory stream... and compressing");
					void[] zipped = std.zlib.compress(cast(void[])m.data(),1);
					fileZipped.write(cast(ubyte[])zipped);
				}
				else
				{
					File file = new File;
					file.create(GlobalSetup.inputfile~".LRtbl",FileMode.Out);
					this.toStream(file);				
				}
			}
			catch(Exception e)
			{
				printf("\nParser::this::Exception while saving: #%.*s#",e.msg); 			
			}
		}
	}
	package class Tabs
	{
		void print(Stream file)
		{
			file.printf("\n");
			for(int i=0;i<num;i++)
			{
				file.printf("\t");
			}
		}
		int opPostInc ( )  {  return num++; }
		int opPostDec ( )  {  return num--;	}
		void opCall (Stream file)  {  print(file); }
		private int num = 0;
	}
	void generate()
	{
		Tabs tabs = new Tabs;
		File file = new File;
		file.create(this.GlobalSetup.inputfile[0..length-5]~".d",FileMode.Out);
    
    file.printf("%.*s\n",this.G.mainCode);
		
    file.printf("\nclass SyntaxTreeParser\n{");
		
		GrammarRule[] prods;
		tabs++;
		tabs(file);
		file.printf("static bool cmpProdAndNodeArray(int[] prod, ILENode[] na)");
		tabs(file);
		file.printf("{");
		tabs++;
		tabs(file);
		file.printf("if(prod.length!=na.length)return false;");
		tabs(file);
		file.printf("for(int i=0;i<prod.length; i++){if(prod[i]!=na[i].element.getIndex())return false;}return true;");
		tabs--;
		tabs(file);
		file.printf("}");
		foreach(char[] name,int x; this.G.nonTerminalToInt)
		{
			prods = this.G.getProductionsByLeftSide(x);
			tabs(file);
			if(prods.length>=1)
			{
				if(prods[0] in this.G.rule2Code && this.G.rule2Code[prods[0]].arguments!="")
				{
					file.printf("static void %.*s(inout ILENode[] nodes,%.*s)",name[1..name.length-1],this.G.rule2Code[prods[0]].arguments);
				}
				else
				{
					file.printf("static void %.*s(inout ILENode[] nodes)",name[1..name.length-1]);
				}
			}
			else
			{
				throw new ParsingException("Something wrong with this grammar, no rule for nonterminal "~name~"?");
			}
			tabs(file);
			file.printf("{");
			tabs++;
			tabs(file);
			file.printf(r"//%.*s ima %d podGrammarRule",name[1..name.length-1],prods.length);
			{
				foreach(inout GrammarRule prod; prods)
				{
					tabs(file);
					file.printf("/* %.*s */ ",prod.toString(G));
					tabs(file);
					file.printf("{");tabs++;tabs(file);
					file.printf("static int[] prod=[");
					foreach(int x; prod.right){file.printf("%d,",x);}
					file.printf("];");
					tabs(file),file.printf("if(cmpProdAndNodeArray(prod,nodes))");
					tabs(file),file.printf("{");
					tabs++,tabs(file);
					{
						for(int i=0;i<prod.length;i++)
						{
							if(prod in this.G.rule2Code)
							{
								GrammarRuleCode grc = this.G.rule2Code[prod];
								tabs(file);
                foreach(char[] line;grc.parts[i])
								{
									file.printf("%.*s",line);
								}
							}
							if(prod.right[i]>0)//nezavrsni
							{
									tabs(file);
									if(prod in this.G.rule2Code && this.G.rule2Code[prod].calls[i]!="")
									{
										GrammarRuleCode grc = this.G.rule2Code[prod];
										file.printf("%.*s(nodes[%d].subNodes,%.*s);",G.getZnak(prod.right[i])[1..length-1],i,grc.calls[i]);
									}
									else
									{
										file.printf("%.*s(nodes[%d].subNodes);",G.getZnak(prod.right[i])[1..length-1],i);
									}
							}
						}
						if(prod in this.G.rule2Code)
						{
							GrammarRuleCode grc = this.G.rule2Code[prod];
              tabs(file);
							foreach(char[] line;grc.parts[prod.length])
							{
								file.printf("%.*s",line);
							}
						}
					}
					tabs--,tabs(file),file.printf("}"); 				
					tabs--;tabs(file);file.printf("}");
				}
			}
			tabs--;
			tabs(file);
			file.printf("}");
		}
		tabs--;
		file.printf("\n}");
		
		//file.printf("\nint main(char [][] args)\n{\n\treturn 0;\n}");
		file.close();
	}


	bool tableFromStream(Stream file)
	{
		printf("\nLoading table from file... ");
		int tmp;
		file.read(tmp);
		tablica.length = tmp;
		file.read(tmp);
		foreach(inout RowElementSet[] a; tablica)
		{
			a.length = tmp;
		}

		int i,j,k;

		do
		{
			file.read(i);
			file.read(j);
			file.read(k);
			debug writef("%d x %d -> %d\t",i,j,k);
			if(tablica[i][j] is null)
			{
				tablica[i][j] = new RowElementSet();
			}

			for(int pom=0; pom<k; pom++)
			{
				tablica[i][j].insert(new RowElement(file));
			}
		}

		while(!file.eof);

		printf(" there are %d states.",tablica.length);

		file.close();
		return true;
	}

	void FillZnakToIndex()
	{
		int broj=0;
		foreach(char[] i,int br; G.terminalToInt)
		{
			if(br != Special.TOCKA)
			{
				znakToIndex[br]=broj;
				broj++;
			}
			debug writefln("TOKENTOINDEX:: %s \t %s \t %s",i,br,broj);
		}
		foreach(char[] i,int br; G.nonTerminalToInt)
		{
			znakToIndex[br]=broj;
			debug writefln("TOKENTOINDEX:: %s \t %s \t %s",i,br,broj);
			broj++;
		}
	}

	void KonstruirajTablicu()
	{
		tablica.length = Stanja.length;
		int len = znakToIndex.length;
		foreach(inout RowElementSet[] a; tablica)
		{
			a.length = len;
		}

		int pom;
		foreach(State State;Stanja.Values())
		{
			foreach(Transititon p; State.pri)
			{
				pom=znakToIndex[p.znak];
				if(tablica[State.BR][pom] is null)
				{
					tablica[State.BR][pom] = new RowElementSet();
				}
				tablica[State.BR][pom].insert(new RowElement(p.brstanja,0,G));
			}
		}
		foreach(State State; Stanja.Values())
		{
			foreach(Stavka stavka; State.stavke.Values())
			{
				if(stavka.prod.right.length == stavka.prod.PolozajTocke)
				{
					foreach(int znak; stavka.lookahead.Values())
					{
						pom=znakToIndex[znak];
						if(tablica[State.BR][pom] is null)
						{
							tablica[State.BR][pom] = new RowElementSet();
						}
						else
						{
							if(GlobalSetup.parserOutput)
								printf("\nAmbiguous %.*s, state %d",G.terminalToString[znak],State.BR);
						}
						tablica[State.BR][pom].insert(new RowElement(Special.REDUCTION,stavka.prod.right.length,stavka.prod.left,G));
					}
				}
			}
		}
		pom = znakToIndex[G.nonTerminalToInt["<S_>"]];

		if(tablica[0][pom] is null)
		{
			tablica[0][pom] = new RowElementSet();
		}

		tablica[0][pom].insert(new RowElement(0,0,G));
		pom = znakToIndex[Special.EOF];

		if(tablica[0][pom] is null)
		{
			tablica[0][pom] = new RowElementSet();
		}

		tablica[0][pom].insert(new RowElement(Special.ACCEPT,0,G));
	}

	public void IspisiStanja()
	{
		foreach(State st; Stanja.Values())
		{
			printf("\n\n%.*s",st.toString);
		}
	}

	void IzracunajStanja()
	{
		clock_t pocetak = clock();

		State s = new State();

		foreach(GrammarRule p; G.grammarRules.Values())
		{
			if(p.left == G.parsersFirstNonTerminal)
			{
				GrammarRule prod = GrammarRule.newGrammarRuleWithDot(p,0);
				s.stavke.insert(new Stavka(G,prod,Special.EOF));
			}
		}
		s.ProsiriState();
		this.Stanja.insert(s);
		int[]znakovi;
		int size,mastersize;

		//IZRACUNAJ STANJA...
		int pass=0;
		do
		{
			pass++;
			StateSet nova = new StateSet();
			StateSet stara = new StateSet(this.Stanja);
			mastersize = stara.length;
			
			bool stanja_contains, nova_contains, stara_contains;
			printf("\nStates  Current    New      Time");
			
			do
			{
				size = Stanja.length;
				printf("\n");
				foreach(int index,State st; stara.Values())
				{
					znakovi ~= 4;
					st.ZnakoviIzaTocke(znakovi);
					foreach(int znak; znakovi)
					{
						State State = st.IzracunajStateZaZnak(znak);
						if(G.GlobalSetup.lalr)
						{
							stanja_contains = Stanja.find(State);
							nova_contains = nova.find(State);
							stara_contains = stara.find(State);
							//if(!stanja_contains)Stanja.insert(State);
							if(!stanja_contains && !nova_contains && !stara_contains)
							{
								nova.insert(State);	
							}
							//else
							{
								if(stanja_contains || stara_contains || nova_contains)
								{
									State.StateCounter--;
								}
							}
						}
						else
						{
							nova_contains = nova.find(State);
							stara_contains = stara.find(State);
							stanja_contains = Stanja.find(State);
							if(!stanja_contains && !nova_contains && !stara_contains)
							{
								nova.insert(State);	
							}
							else
							{
								if(stanja_contains || stara_contains || nova_contains)
								{
									State.StateCounter--;
								}
							}
						}
						printf("%6d  %6d:%7d%10d ms\r\r",Stanja.length,index,nova.length,clock()-pocetak);
						Transititon newTrans = new Transititon(znak,State.BR);
						bool transFound=false;
						foreach(Transititon tr; st.pri)
						{
							if(tr==newTrans)transFound=true;
						}
						if(transFound==false)
						{
							st.pri ~= new Transititon(znak,State.BR);
						}
					}
				}
				Stanja.insert(nova);
				if(G.GlobalSetup.lalr)
				{
					//stara.insert(Stanja);
				}
	
				stara = nova.dup;
	
				nova.clear();
			}
			while(size != Stanja.length);
		}
		while(GlobalSetup.lalr ? pass<GlobalSetup.lalrExtraPasses+1 : false)

		printf("\nCalculating states took: %dms",clock()-pocetak);

		printf("\nTotal %d states",Stanja.length);
/*
		foreach(int ind, State st; Stanja.Values())
		{
			if(st.BrojNeREDUCTION != st.pri.length)
			{
				printf("\n%.*s",st.toString);
			}
			assert(st.BrojNeREDUCTION == st.pri.length);
		}
*/		
	}
	
	version(enableHTMLoutput)
	{
		void TablicaToFile()
		{
			File file = new File();
			file.create(GlobalSetup.inputfile~".html",FileMode.Out);
			file.printf("<html><head><title>Tablica LR(1) parsera</title></head");
			file.printf("<body>Tablica LR(1) parsera");
			foreach(GrammarRule p; G.grammarRules.Values())
			{
				file.printf("<br>");
				p.toHTML(file,G);
			}
	
			file.printf("</body>");
			foreach(State st; Stanja.Values())
			{
				st.toHTML(file);
			}
	
			file.printf("<table border><tr><td></td>");
	
			for(int i=0; i<G.terminalToInt.length + G.nonTerminalToInt.length-1; i++)
				file.printf("<td>%d</td>",i);
	
			file.printf("</tr><tr><td></td>");
	
			foreach(char[] i,int br; G.terminalToInt)
			{
				if(br != Special.TOCKA)
					file.printf("<td>%.*s</td>",i);
			}
	
			foreach(char[] i,int br; G.nonTerminalToInt)
			{
				file.printf("<td>&lt%.*s&gt</td>",i[1..i.length-1]);
			}
	
			file.printf("</tr>");
			int br =0;
			foreach(int i,RowElementSet[] redak; tablica)
			{
				file.printf("<tr><td>%d</td>",i);
				foreach(RowElementSet elems; redak)
				{
					file.printf("<td><table>");
	
					if(!(elems is null))
					{
						foreach(RowElement e; elems.Values())
						{
							file.printf("<tr>");
							e.toHTML(file);
							file.printf("</tr>");
						}
					}
					else
					{}
					file.printf("</table></td>");
				}
				file.printf("</tr>");
			}
			file.printf("</table>");
			file.printf("</html>");
			file.close();
		}
	}

	bool DaLiVecPostojiOvajParser(ParserDevice p, ParserDevice[] pars)
	{
		foreach(ParserDevice xx; pars)
		{
			if(xx == p)return true;
		}
		return false;
	}

	bool Parse(ILexElement[] u)
	{
		debug writefln("\nParser::Parse(ILexElement[])");
		resetParser();
		if(u.length<1)throw new ParsingException("Too short input line!");
		u ~= u[0].create("EOF");
		ulazI = u;
		
		parsers ~= new ParserDevice(this,0,0);
		
		debug
		{
			writefln("in Parse...");
			foreach(ILexElement ile; ulazI)
			{
				writef("%s ",ile.getIndex);
			}
		}
		
		bool result;
		ParserDevice[] novi,svinovi,svi;
		bool accepted = false;
		
		do
		{
			result = false;
			debug writefln("result=%d",result," novi=%d, svinovi=%d, svi=%d",novi.length,svinovi.length,svi.length);
			foreach(ParserDevice pd; parsers)
			{
				debug writefln("parserdevice(znak=%d, State=%d, stog.length=%d)",pd.trenutnizn,pd.State,pd.stog.length);
				try
				{				
					pd.BranchI(novi);
					debug writefln("nakon BranchI novi=%d",novi.length);
					pd.PrintI();
					result |= pd.OneStepI(0);
				}
				catch(ParsingException e)
				{
					printf("\n%.*s",e.msg);
					return false;	
				}
				accepted |= pd.accepted;

				if(pd.accepted==true)this.finished~=pd;
				if(pd.finished==true)svi~=pd;
				foreach(int index, ParserDevice pd2; novi)
				{
					pd2.PrintI();
					try
					{
						pd2.OneStepI(index+1);
					}
					catch(ParsingException e)
					{
						printf("\n%.*s",e.msg);
						return false;	
					}
				}
				foreach(ParserDevice p; novi)
				{
					svinovi~=p;
				}
				novi.length = 0;
			}
			foreach(int index, ParserDevice pd2; svinovi)
			{
				if(!pd2.finished && !DaLiVecPostojiOvajParser(pd2,parsers) && !DaLiVecPostojiOvajParser(pd2,svi))
					parsers ~= pd2;
			}
			svinovi.length = 0;
			foreach(ParserDevice pd2; parsers)
			{
				if(!pd2.finished && !DaLiVecPostojiOvajParser(pd2,svinovi)&& !DaLiVecPostojiOvajParser(pd2,svi))
				{
					svinovi~=pd2;
				}
			}
			if(accepted)break;
			parsers = svinovi.dup;
			svinovi.length = 0;
		}
		while(parsers.length != 0);

		if(accepted)
		{
			printf("\nACCEPTED!");
			return true;
		}
		else
		{
			printf("\nREJECTED!");
			return false;
		}
	}

	bool Parse(char[] u,SplitType splitmode=SplitType.SpaceSplit)
	{
		if(splitmode==SplitType.SpaceSplit)
		{
			return Parse(split(u));
		}
		else 
		if(splitmode==SplitType.CharByCharSplit)
		{
			char[][] input = new char[][u.length];
			foreach(int index,char inchar; u)
			{
				input[index] ~= inchar;
			}
			return Parse(input);
		}
		else throw new ParsingException("Unknown string spliting mode!");
	}
	bool Parse(char[][] u)
	{
		resetParser();
		class LexElem: ILexElement
		{
			this(char[] znak)
			{
				this.index = G.getIndex(znak);
			}	
			int getIndex()
			{
				return index;
			}
			ILexElement create(char[] name)
			{
				return new LexElem(name);
			}
			private int index;
			static Grammar G;
		}
		//fix (next line) isn't really thread safe
		LexElem.G = G;
		ILexElement[] input = new ILexElement[u.length];
		foreach(int index,char[] inputToken; u)
		{
			input[index] = new LexElem(inputToken);
		}
		return this.Parse(input);
	}

	bool toStream(Stream file)
	{
		file.write(tablica.length);
		file.write(tablica[0].length);

		for(int i=0; i<tablica.length; i++)
		{
			for(int j=0; j<tablica[i].length; j++)
			{
				if(!(tablica[i][j] is null))
				{
					file.write(i);
					file.write(j);
					file.write(tablica[i][j].length);

					for(int k=0; k<tablica[i][j].length; k++)
					{
						tablica[i][j][k].toStream(file);
					}
				}
			}
		}
		return true;
	}
	private void resetParser()
	{
		int trenutnizn = 0;
		parsers.length=0;
		finished.length=0;
		ulaz.length=0;
		ulazI.length=0;
		ParserDevice.pdID = 0;
	}

	public Grammar grammar(){return G;}
	public StateSet states(){return Stanja;}

	public ParserDevice[] parsers;
	public ParserDevice[] finished;
	private CharArraySet proturjecnosti; //not currently populated, needs a fix
	protected Setup GlobalSetup;
	protected Grammar G;
	//union
	private
	{
		private char[][] ulaz;//not used any more, everything is done with ILexElement arrays
		private ILexElement[] ulazI;
	}

	private int[int] znakToIndex;
	private StateSet Stanja;
	private Table tablica;
}

//čćšđžČĆŠĐŽ

