﻿/*
This is the main test program of the parser

writen by Ivan Senji
*/

module lrparsermain;
private import lrparsermain_bn;

import lr;
import std.stream;
import std.string;
import std.stdio;
import std.c.time;

class MyLex: ILexElement
{
	this(char[] znak)
	{
		this.znak = znak;
		this.index = G.getIndex(this.znak);
	}	
	int /*ILexElement.*/getIndex()
	{
		return index;
	}
	ILexElement /*ILexElement.*/create(char[] name)
	{
		return new MyLex(name);
	}
	private char[] znak;
	private int index;
	static Grammar G;
}

void TestParser(Setup GlobalSetup)
{
	alias Parser.Set Set;
	Parser P2;
	if(GlobalSetup.parserOutput)
	{
		if(GlobalSetup.forseRecalc)
		{
			if(GlobalSetup.lalr)
				P2 = new Parser(Set.LalrExtra,GlobalSetup.lalrExtraPasses,Set.Tree2XML,Set.ParserOutput,Set.Input,GlobalSetup.inputfile,Set.Recalc,Set.Lalr);
			else 
				P2 = new Parser(Set.Tree2XML,Set.ParserOutput,Set.Input,GlobalSetup.inputfile,Set.Recalc);
		}
		else
		{
			if(GlobalSetup.lalr)
				P2 = new Parser(Set.LalrExtra,GlobalSetup.lalrExtraPasses,Set.Tree2XML,Set.ParserOutput,Set.Input,GlobalSetup.inputfile,Set.Lalr);
			else
				P2 = new Parser(Set.Tree2XML,Set.ParserOutput,Set.Input,GlobalSetup.inputfile);
		}
	}
	else
	{
		if(GlobalSetup.forseRecalc)
		{
			if(GlobalSetup.lalr)
				P2 = new Parser(Set.LalrExtra,GlobalSetup.lalrExtraPasses,Set.Tree2XML,Set.Input,GlobalSetup.inputfile,Set.Recalc,Set.Lalr);
			else 
				P2 = new Parser(Set.Tree2XML,Set.Input,GlobalSetup.inputfile,Set.Recalc);
		}
		else
		{
			if(GlobalSetup.lalr)
				P2 = new Parser(Set.LalrExtra,GlobalSetup.lalrExtraPasses,Set.Tree2XML,Set.Input,GlobalSetup.inputfile,Set.Lalr);
			else
				P2 = new Parser(Set.Tree2XML,Set.Input,GlobalSetup.inputfile);
		}
	}
	MyLex.G = P2.grammar;
	
	if(!(GlobalSetup.testfile is null) && !(GlobalSetup.inputfile is null))
	{
		File file = new File(GlobalSetup.testfile,FileMode.In);
		char[][] words;
		MyLex[] wordsLex;
		char[]line;
		clock_t pocetak;
		
		if(GlobalSetup.testFileLoadMethod==0)
		{
			do
			{
 				line = file.readLine();
 				pocetak = clock();
				P2.Parse(line);
 				writefln("\t1time %dms, parsers finished %d",clock()-pocetak,P2.finished.length);
			}while(!file.eof)
		}
		else if(GlobalSetup.testFileLoadMethod==1)
		{
			do
			{
				line = file.readLine();
				words ~= split(line);
			}while(!file.eof)
			writefln("\n\nParsing2: ");
			foreach(char[] part; words)
			{
				writefln(part);
				wordsLex~=new MyLex(part);
			}
			pocetak = clock();
			P2.Parse(castA2I!(MyLex,ILexElement)(wordsLex));
			writefln("\t2time %dms, parsers finished %d",clock()-pocetak,P2.finished.length);
		}
		else return;
	}
	P2.generate();
}
void TestParser2()
{
	alias Parser.Set Set;
	alias Parser.SplitType.CharByCharSplit splitMode;
	Parser p1 = new Parser(Set.Input,"tests\\test01.gram");
	writefln("Result parsing # %.*s # is %d","12345",p1.Parse("12345",splitMode));
	writefln("Result parsing # %.*s # is %d","123.4345",p1.Parse("123.4345",splitMode));
	writefln("Result parsing # %.*s # is %d","12t3.4345",p1.Parse("12t3.4345",splitMode));
	writefln("Result parsing # %.*s # is %d","124.",p1.Parse("124.",splitMode));
}

/* =================MAIN==============*/
int main ( char [] [] args )
{
	writefln("\nIvan's LR(1)/LALR(1) parser! Build ", auto_build_number);
	//TestParser2();
	TestParser(ParseArguments(args));
	return 1;
}

Setup ParseArguments(char [] [] args)
{
	Setup GlobalSetup = new Setup();
	for(int i=1; i<args.length; i++)
	{
		switch(args[i])
		{
			case "-i":
			case "-I":
				if(i+1 <= args.length-1)
				{
					GlobalSetup.inputfile = args[i+1];
					i++;
				}
				break;
			case "-test":
				if(i+1 <= args.length-1)
				{
					GlobalSetup.testfile = args[i+1];
					GlobalSetup.testFileLoadMethod = 0;
					i++;
				}			
				break;
			case "-test2":
				if(i+1 <= args.length-1)
				{
					GlobalSetup.testfile = args[i+1];
					GlobalSetup.testFileLoadMethod = 1;
					i++;
				}			
				break;
			case "+lalr":
				GlobalSetup.lalr = true;
				if(i+1 <= args.length-1)
				{
					GlobalSetup.lalrExtraPasses=atoi(args[i+1]);
					i++;
				}
				else
					writefln("Missing passses!");
				break;
			case "+prod":
				GlobalSetup.printProductions = true;
				break;
			/*case "-wait":
				GlobalSetup.wait = true;
				break;*/
			case "-po":
				GlobalSetup.parserOutput = false;
				break;
			case "+po":
				GlobalSetup.parserOutput = true;
				break;
			case "+nez":
				GlobalSetup.printNez = true;
				break;
			case "+zav":
				GlobalSetup.printZav = true;
				break;
			/*case "+parsetree":
				GlobalSetup.printParseTree = true;
				break;*/
			case "+parsetree2XML":
				GlobalSetup.printParseTree2XML = true;
				break;
			case "+forseRecalc":
				GlobalSetup.forseRecalc = true;
				break;
			default: 
				printf("\nUnknown command line argument: %.*s",args[i]);
				break;
		}
	}
	if(GlobalSetup.inputfile is null && GlobalSetup.testfile is null)
	{
		writefln(
"\nUsage: command line arguments:
-i\t\tmust be followed with the file containing grammar
-test\t\tmust be folowed with the file containig test, one line - one test
-test2\t\tmust be folowed with the file containig test, whole file - one test
+prod\t\tprints all productions to screen
-wait\t\twaits for a key press after finishing
+po\t\tprints parser output step by step
+nez\t\tprints all nonterminal symbols to screen
+zav\t\tprints all terminal symbols to screen
+parsetree\tprints parse tree to screen
+parsetree2XML\toutputs the parse tree to XML file
+forseRecalc\tforse to recalcualte the table and  not just load it from file
+gen\t\tgenerate .d file with parser
Example:
	LR -i gramatika3.gram -test2 gramatika3.test.txt +parsetree2XML
	-loads the grammar from 'gramatika3.gram', creates a parse table and
	tests the contents of 'gramatika3.test.txt', and outputs the result to 
	XML file
");
		return null;
	}
	else
	return GlobalSetup;
}


//AšAŠAđAĐAčAČAćAĆAžAŽA  CcGgHhUujJSs
